import os
import json5
import shutil
import MarkdownParser
import traceback
import sys
import syntaxlight
import types
from .util import *
from .zood import (
    parse_config,
    caculate_front_next_url,
    get_dir_tree,
    url_replace,
)

LANGUAGE_USED = set()
TOTAL_ERROR_NUMBER = 0
CODE_BLOCK_NUMBER = 1

def generate_web_docs(md_dir_name):
    directory_tree, markdown_htmls = parse_markdown(md_dir_name)
    # directory_tree 为当前项目的目录树
    # markdown_htmls 为markdown 文档对应的 html
    generate_docs(directory_tree, markdown_htmls, md_dir_name)


def parse_markdown(md_dir_name):
    current_dir = os.getcwd()
    dir_yml_path = os.path.join(current_dir, md_dir_name, "dir.yml")
    dir_yml = read_configfile(dir_yml_path)
    yml_sort(dir_yml)
    directory_tree = []
    markdown_htmls = {}
    markdown_parser = MarkdownParser.Markdown()

    # 将错误信息重定向到 error.log 中
    error_log_file = open(os.path.join(os.path.dirname(__file__), "config", "error.log"), "w", encoding="utf-8")
    sys.stderr = error_log_file

    for dir_name, files in dir_yml.items():
        file_names = []
        for i in files:
            file_name = list(i.keys())[0]
            file_path = os.path.join(md_dir_name, dir_name, file_name + ".md")
            if not os.path.exists(file_path):
                print_info("[zood解析失败]: 找不到文件" + file_path)
                print("如手动删除md文件可使用 zood update 更新 dir.yml")
                exit(1)
            else:
                with open(file_path, "r", encoding="utf-8") as f:
                    lines = markdown_parser.preprocess_parser(f.read())
                    root = markdown_parser.block_parser(lines)
                    tree = markdown_parser.tree_parser(root)
                    hightlight_codeblock(tree, file_path)
                    header_navigater = markdown_parser.get_toc(tree)
                    markdown_html = tree.toHTML(header_navigater)

                markdown_htmls[file_path] = markdown_html
                file_names.append(file_name)
        directory_tree.append({dir_name: file_names})

    error_log_file.close()

    global TOTAL_ERROR_NUMBER
    if TOTAL_ERROR_NUMBER != 0:
        print_info(f"[zood]: 代码段解析出现 {TOTAL_ERROR_NUMBER} 处错误, 已跳过高亮解析, 使用 zood log 查看错误信息")

    return directory_tree, markdown_htmls


def hightlight_codeblock(tree: MarkdownParser.Block, file_path: str):
    """
    使用 syntaxlight 更新其中代码段的高亮
    https://github.com/luzhixing12345/syntaxlight
    """

    def code_to_html(self):
        return f'<pre class="language-{self.input["language"]}"><code>{self.input["code"]}</code></pre>'

    def pic_to_html(self):
        word = self.input["word"]
        url = self.input["url"]
        return f'<a data-lightbox="example-1" href="{url}"><img src="{url}" alt="{word}"></a>'

    global LANGUAGE_USED
    global TOTAL_ERROR_NUMBER
    global CODE_BLOCK_NUMBER
    for block in tree.sub_blocks:
        if block.block_name == "CodeBlock":
            language = block.input["language"]
            if syntaxlight.is_language_support(language) or language == 'UNKNOWN':
                if language == "UNKNOWN":
                    language = 'txt'
                else:
                    language = syntaxlight.clean_language(language)
                block.input["language"] = language
                try:
                    code = syntaxlight.parse(block.input["code"], language, file_path)
                except Exception as e:
                    code = None
                    print_info(f"syntaxlight error exit: {e}")
                    print(f"This is a bug, please issue in: https://github.com/luzhixing12345/syntaxlight/issues\n")
                    traceback.print_exc()
                if code is not None:
                    block.input["code"] = code
                    block.to_html = types.MethodType(code_to_html, block)
                    LANGUAGE_USED.add(language)
                else:
                    TOTAL_ERROR_NUMBER += 1
                CODE_BLOCK_NUMBER += 1
        elif block.block_name == "PictureBlock":
            block.to_html = types.MethodType(pic_to_html, block)
        else:
            hightlight_codeblock(block, file_path)


def generate_docs(directory_tree, markdown_htmls, md_dir_name):
    config = get_zood_config()
    html_dir_name = config["html_folder"]

    index_README_path = os.path.join(md_dir_name, ".", "README.md")
    if not os.path.exists(index_README_path):
        print_info(f"您需要保留 {md_dir_name}/README.md 作为首页")
        exit()

    if os.path.exists(html_dir_name):
        shutil.rmtree(html_dir_name)
    os.makedirs(os.path.join(html_dir_name, "articles"))
    os.makedirs(os.path.join(html_dir_name, "js"))
    os.makedirs(os.path.join(html_dir_name, "css"))
    os.makedirs(os.path.join(html_dir_name, "img"))

    # 复制图片
    imgs_path = os.path.join(os.path.dirname(__file__), "config", "img")
    for root, _, files in os.walk(imgs_path):
        for img in files:
            shutil.copy(os.path.join(root, img), os.path.join(html_dir_name, "img"))

    flat_paths = []  # 扁平化之后的所有文件的路径
    for item in directory_tree:
        dir_name = list(item.keys())[0]
        files = item[dir_name]
        if dir_name == ".":
            dir_name = md_dir_name
        for file in files:
            flat_paths.append(os.path.join(dir_name, file))

    # 目录树
    dir_tree_html = get_dir_tree(directory_tree, md_dir_name)
    front_url, next_url = caculate_front_next_url(flat_paths, index_README_path, md_dir_name)
    github_icon = get_github_icon(config["options"]["enable_github"])

    global LANGUAGE_USED
    LANGUAGE_USED = list(LANGUAGE_USED)
    LANGUAGE_USED.sort()
    syntaxlight.export_css(LANGUAGE_USED, os.path.join(html_dir_name, "css"))

    hightlight_css = ""
    for l in LANGUAGE_USED:
        hightlight_css += f"<link rel='stylesheet' href=../../../css/{l}.css />"

    html_template = parse_config(config, markdown_htmls)
    index_html_path = os.path.join(html_dir_name, "index.html")
    html_template = html_template.replace("hightlight-css", hightlight_css)
    with open(index_html_path, "w", encoding="utf-8") as f:
        # index 的地址做一些修改
        index_html_template = html_template.replace("../../.", "")
        next_url = next_url.replace("../..", "./articles")
        index_dir_tree_html = dir_tree_html.replace("../..", "./articles")
        index_html_template = index_html_template.replace("../..", "./articles")

        index_html_template = index_html_template.replace("directory-tree-scope", index_dir_tree_html)
        index_html_template = index_html_template.replace("github-icon", github_icon)
        index_html_template = url_replace(index_html_template, front_url, next_url, "b")
        f.write(index_html_template.replace("html-scope", markdown_htmls[index_README_path]))

    html_template = html_template.replace("directory-tree-scope", dir_tree_html)
    html_template = html_template.replace("github-icon", github_icon)
    for file_path, markdown_html in markdown_htmls.items():
        dir_name = file_path.split(os.sep)[1]
        file_name = file_path.split(os.sep)[2].replace(".md", "")
        if dir_name == ".":
            dir_name = md_dir_name
        doc_path = os.path.join(html_dir_name, "articles", dir_name, file_name)
        os.makedirs(doc_path)
        html_path = os.path.join(doc_path, "index.html")
        with open(html_path, "w", encoding="utf-8") as f:
            front_url, next_url = caculate_front_next_url(flat_paths, file_path, md_dir_name)
            # print(html_path,front_url,next_url)
            final_html = url_replace(html_template, front_url, next_url, "ab")
            f.write(final_html.replace("html-scope", markdown_html))

    print_info(f"[zood]: 已生成 {html_dir_name}/", color="green")

    # 对于 vscode 的 live server 的优化
    vscode_settings = os.path.join(".vscode", "settings.json")

    if os.path.exists(vscode_settings):
        with open(vscode_settings, "r", encoding="utf-8") as f:
            settings = json5.load(f)
            if "liveServer.settings.port" in settings:
                print(
                    f'\nVscode live server: http://127.0.0.1:{settings["liveServer.settings.port"]}/docs/index.html\n'
                )
    else:
        print(f"\nVscode live server: http://127.0.0.1:5500/docs/index.html\n")



"""
这个文件的作用是更新 PYPI 包, 修改 pyproject.toml 的 VERSION 版本号并发布, 相当于 poetry build + poetry publish

使用方法

    python update.py                  发布版本更新
    python update.py sub              次版本更新
    python update.py main             主版本更新

"""

import shutil
import re
import os
import json
import subprocess

def print_info(msg, color="red"):
    if color == "red":
        print(f"\033[1;31m{msg}\033[0m")
    elif color == "green":
        print(f"\033[1;32m{msg}\033[0m")
    else:
        print("未知 color: (red, green)")


def update_PYPI_package(choice=None):
    """
    choice = None(default) 发布版本更新
    choice = sub           次版本更新
    choice = main          主版本更新
    """

    pwd = os.getcwd()
    pyproject_path = os.path.join(pwd, "pyproject.toml")
    if not os.path.exists(pyproject_path):
        print("pyproject.toml 不存在")
        return

    try:
        shutil.rmtree("dist")
        print("delete dist")
    except:
        pass

    with open(pyproject_path, "r", encoding="utf-8") as f:
        file = f.read()

    old_version_re = re.search(r"version = \"(\d+)\.(\d+)\.(\d+)\"", file)
    MAIN_VERSION, SUB_VERSION, FIX_VERSION = (
        int(old_version_re.group(1)),
        int(old_version_re.group(2)),
        int(old_version_re.group(3)),
    )

    old_version = f"{MAIN_VERSION}.{SUB_VERSION}.{FIX_VERSION}"

    if choice is None:
        FIX_VERSION = FIX_VERSION + 1
    elif choice == "sub":
        SUB_VERSION = SUB_VERSION + 1
        FIX_VERSION = 0
    elif choice == "main":
        MAIN_VERSION = MAIN_VERSION + 1
        SUB_VERSION = 0
        FIX_VERSION = 0

    new_version = f"{MAIN_VERSION}.{SUB_VERSION}.{FIX_VERSION}"

    new_version_str = f'version = "{MAIN_VERSION}.{SUB_VERSION}.{FIX_VERSION}"'
    new_content = re.sub(r"version = \"(\d+)\.(\d+)\.(\d+)\"", new_version_str, file)
    with open(pyproject_path, "w", encoding="utf-8") as f:
        f.write(new_content)

    print(f"{old_version} -> {new_version}")

    result = subprocess.run(['poetry', 'build'], capture_output=True, text=True)
    if result.returncode != 0:
        print_info("库更新失败, pyproject.toml 版本已回退")
        with open(pyproject_path, "w", encoding="utf-8") as f:
            f.write(file)
        return
    result = subprocess.run(['poetry', 'publish'], capture_output=True, text=True)
    if result.returncode != 0:
        print_info("库更新失败, pyproject.toml 版本已回退")
        with open(pyproject_path, "w", encoding="utf-8") as f:
            f.write(file)
        return
    print_info('更新成功','green')


def update_vsce_package(choice=None):
    
    if not os.path.exists('package.json'):
        print('package.json 不存在')
        return

    with open('package.json', 'r') as f:
        file = json.load(f)
    version = file['version']
    major, minor, patch = map(int, version.split('.'))
    old_version = f'{major}.{minor}.{patch}'

    if choice is None:
        patch += 1
    elif choice == 'sub':
        minor += 1
        patch = 0
    else:
        major += 1
        minor = 0
        patch = 0
    
    new_version = f'{major}.{minor}.{patch}'
    print(f"{old_version} -> {new_version}")

    result = subprocess.run(['pnpm', 'vsce','package','--no-dependencies'], capture_output=True, text=True)
    if result.returncode != 0:
        print_info("库更新失败, package.json 版本已回退")
        with open('package.json', "w", encoding="utf-8") as f:
            f.write(file)
        return
    result = subprocess.run(['pnpm', 'vsce','publish','--no-dependencies'], capture_output=True, text=True)
    if result.returncode != 0:
        print_info("库更新失败, pyproject.toml 版本已回退")
        with open('package.json', "w", encoding="utf-8") as f:
            f.write(file)
        return
    print_info('更新成功','green')


import yaml
import os
import re
import json
import subprocess
from typing import Dict, List


def read_configfile(file_path: str) -> Dict[str, List[Dict[str, int]]]:
    if not os.path.exists(file_path):
        print_info("找不到文件" + file_path)
        exit(0)

    with open(file_path, "r", encoding="utf-8") as f:
        file_data = f.read()

    data = yaml.load(file_data, Loader=yaml.FullLoader)
    return data


def writeConfigFile(data, file_path):
    with open(file_path, "w", encoding="utf-8") as f:
        yaml.dump(data, f, allow_unicode=True, sort_keys=False)


def yml_sort(yml):
    for _, files in yml.items():
        files.sort(key=lambda item: list(item.values())[0])


def print_info(msg, color="red"):
    if color == "red":
        print(f"\033[1;31m{msg}\033[0m")
    elif color == "green":
        print(f"\033[1;32m{msg}\033[0m")


def get_zood_config():
    """
    获取基础配置信息, 优先查找当前目录下 md-docs/ 是否存在 _config.yml
    如果本地不存在则使用全局 _config.yml 作为配置信息
    """
    global_config_path = os.path.join(os.path.dirname(__file__), "config", "_config.yml")

    global_zood_config = read_configfile(global_config_path)
    md_dir_name = global_zood_config["markdown_folder"]

    local_config_path = os.path.join(md_dir_name, "_config.yml")
    if os.path.exists(local_config_path):
        config_path = local_config_path
    else:
        config_path = global_config_path

    return read_configfile(config_path)


def caculate_front_next_url(flat_paths: list, path, md_dir_name):
    dir_name = path.split(os.sep)[1]
    file_name = path.split(os.sep)[2].replace(".md", "")
    if dir_name == ".":
        dir_name = md_dir_name
    path = os.path.join(dir_name, file_name)
    pos = flat_paths.index(path)

    front_url = '"."'
    next_url = '"."'

    if pos != 0:
        front_url = htmlRelativeUrl(flat_paths[pos - 1])
    if pos != len(flat_paths) - 1:
        next_url = htmlRelativeUrl(flat_paths[pos + 1])

    return front_url, next_url


def htmlRelativeUrl(url: str):
    new_url = url.replace(os.sep, "/")
    new_url = f'"../../{new_url}"'
    return new_url


def get_dir_tree(directory_tree, md_dir_name):
    tree_html = ""
    for item in directory_tree:
        dir_name = list(item.keys())[0]
        files = item[dir_name]
        if dir_name == ".":
            dir_name = md_dir_name
            for file in files:
                dir_url_link = f"../../{dir_name}/{file}"
                # print(dir_url_link)
                tree_html += treeItem(file, dir_url_link)
        else:
            sub_tree_html = ""
            for file in files:
                dir_url_link = f"../../{dir_name}/{file}"
                # print(dir_url_link)
                sub_tree_html += treeItem(file, dir_url_link)

            first_dir_url_link = f"../../{dir_name}/{files[0]}"
            tree_html += treeItem(dir_name, first_dir_url_link, sub_tree=sub_tree_html)

    # print(tree_html)
    return f'<div class="dir-tree">{tree_html}</div>'


def treeItem(name, dir_url_link, sub_tree=False):
    if sub_tree:
        link = f'<a href="{dir_url_link}" >{name}</a>'
        return f"<ul><li>{link}{sub_tree}</li></ul>"
    else:
        link = f'<a href="{dir_url_link}" >{name}</a>'
        return f"<ul><li>{link}</li></ul>"


def url_replace(html_template, front_url, next_url, control):
    html_template = html_template.replace("<%front_url%>", front_url).replace("<%next_url%>", next_url)
    html_template = html_template.replace("<%control%>", f'"{control}"')
    return html_template


def getAllAPIText(markdown_htmls, search_scope, md_dir_name):
    all_keys = markdown_htmls.keys()
    API_text = {}
    for key in all_keys:
        splited_key = key.split(os.sep)
        dir_name = splited_key[1]

        if len(search_scope) != 0:
            if dir_name not in search_scope:
                continue
        if dir_name == ".":
            dir_name = md_dir_name
        file_name = splited_key[2].replace(".md", "")
        markdown_text = re.sub(r"<.*?>", "", markdown_htmls[key]).replace('"', "").replace("'", "").replace("\\", "")
        path = f"../../{dir_name}/{file_name}"
        API_text[path] = markdown_text

    json_API_text = json.dumps(API_text).replace('"', '\\"').replace("\\n", "")
    return f'"{json_API_text}"'


def get_github_icon(enable_github):
    if enable_github is False:
        return ""
    try:
        output = subprocess.check_output(["git", "config", "--get", "remote.origin.url"]).strip()
        return get_github_repo_url(output.decode("utf-8"))
    except subprocess.CalledProcessError:
        return ""


def get_github_repo_url(url: str):
    if url.startswith("git@"):
        parts = url.split(":")
        if len(parts) == 2:
            username_and_repo = parts[1]
            username, repo = username_and_repo.split("/")
            url = f"https://github.com/{username}/{repo}"

    github_icon = (
        '<a href="'
        + url
        + '" target="_blank" class="github-corner" aria-label="View source on GitHub"><svg width="80" height="80" viewBox="0 0 250 250" style="fill:#151513; color:#fff; position: absolute; top: 0; border: 0; right: 0;" aria-hidden="true"><path d="M0,0 L115,115 L130,115 L142,142 L250,250 L250,0 Z"></path><path d="M128.3,109.0 C113.8,99.7 119.0,89.6 119.0,89.6 C122.0,82.7 120.5,78.6 120.5,78.6 C119.2,72.0 123.4,76.3 123.4,76.3 C127.3,80.9 125.5,87.3 125.5,87.3 C122.9,97.6 130.6,101.9 134.4,103.2" fill="currentColor" style="transform-origin: 130px 106px;" class="octo-arm"></path><path d="M115.0,115.0 C114.9,115.1 118.7,116.5 119.8,115.4 L133.7,101.6 C136.9,99.2 139.9,98.4 142.2,98.6 C133.8,88.0 127.5,74.4 143.8,58.0 C148.5,53.4 154.0,51.2 159.7,51.0 C160.3,49.4 163.2,43.6 171.4,40.1 C171.4,40.1 176.1,42.5 178.8,56.2 C183.1,58.6 187.2,61.8 190.9,65.4 C194.5,69.0 197.7,73.2 200.1,77.6 C213.8,80.2 216.3,84.9 216.3,84.9 C212.7,93.1 206.9,96.0 205.4,96.6 C205.1,102.4 203.0,107.8 198.3,112.5 C181.9,128.9 168.3,122.5 157.7,114.1 C157.9,116.9 156.7,120.9 152.7,124.9 L141.0,136.5 C139.8,137.7 141.6,141.9 141.8,141.8 Z" fill="currentColor" class="octo-body"></path></svg></a><style>.github-corner:hover .octo-arm{animation:octocat-wave 560ms ease-in-out}@keyframes octocat-wave{0%,100%{transform:rotate(0)}20%,60%{transform:rotate(-25deg)}40%,80%{transform:rotate(10deg)}}@media (max-width:500px){.github-corner:hover .octo-arm{animation:none}.github-corner .octo-arm{animation:octocat-wave 560ms ease-in-out}}</style>'
    )
    return github_icon
